#ifndef M6502_H
#define M6502_H

// See cpu.cc for copyrights.

#include "defs.h"
                               /* 6502 status flags:         */
#define	C_FLAG	  0x01         /* 1: Carry occured           */
#define	Z_FLAG	  0x02         /* 1: Result is zero          */
#define	I_FLAG	  0x04         /* 1: Interrupts disabled     */
#define	D_FLAG	  0x08         /* 1: Decimal mode            */
#define	B_FLAG	  0x10         /* Break [0 on stk after int] */
#define	R_FLAG	  0x20         /* Always 1                   */
#define	V_FLAG	  0x40         /* 1: Overflow occured        */
#define	N_FLAG	  0x80         /* 1: Result is negative      */

/** Rd6502()/Wr6502/Op6502() *********************************/
/** These functions are called when access to RAM occurs.   **/
/** They allow to control memory access. Op6502 is the same **/
/** as Rd6502, but used to read *opcodes* only, when many   **/
/** checks can be skipped to make it fast. It is only       **/
/** required if there is a #define FAST_RDOP.               **/
/************************************ TO BE WRITTEN BY USER **/
extern void Wr6502(register fWORD Addr, register fBYTE Value);
extern BYTE Rd6502(register fWORD Addr);
extern BYTE Op6502(register fWORD Addr) CONSTFUN;

extern BYTE Clocks[256];
class m6502cpu
{
public:
    BYTE A,P,X,Y,S;
    PAIR PC;

private:
    fWORD ICount;
    bool IRQPending;
private:
    inline BYTE fetchop()
    {
    	register BYTE i = Op6502(PC.W++);
    	ICount += Clocks[i];
    	return i;
    }
    inline WORD M_LDWORD()
    {
		register WORD Rg = Op6502(PC.W++);
		Rg |= Op6502(PC.W++) << 8;
		return Rg;
    }
    
    /** Addressing Methods ***************************************/
    /** These macros calculate and return effective addresses.  **/
    /*************************************************************/
    inline WORD MC_Ab()	{ return M_LDWORD(); }
    inline WORD MC_Zp()	{ return BYTE(Op6502(PC.W++)); }
    inline WORD MC_Zx()	{ return BYTE(Op6502(PC.W++)+X); }
    inline WORD MC_Zy()	{ return BYTE(Op6502(PC.W++)+Y); }
    inline WORD MC_Ax()	{ return M_LDWORD()+X; }
    inline WORD MC_Ay()	{ return M_LDWORD()+Y; }
    inline WORD MC_Ix() { register WORD K = BYTE(Op6502(PC.W++) + X); return Op6502(K) | (Op6502(K+1) << 8); }
    inline WORD MC_Iy() { register WORD K = Op6502(PC.W++); return (Op6502(K) | (Op6502(K+1) << 8)) + Y; }
    
    /** Reading From Memory **************************************/
    /** These macros calculate address and read from it.        **/
    /*************************************************************/
    inline BYTE MR_Ab()	{ return Rd6502(MC_Ab()); }
    inline BYTE MR_Im()	{ return Op6502(PC.W++); }
    inline BYTE MR_Zp()	{ return Rd6502(MC_Zp()); }
    inline BYTE MR_Zx()	{ return Rd6502(MC_Zx()); }
    inline BYTE MR_Zy()	{ return Rd6502(MC_Zy()); }
    inline BYTE MR_Ax()	{ return Rd6502(MC_Ax()); }
    inline BYTE MR_Ay()	{ return Rd6502(MC_Ay()); }
    inline BYTE MR_Ix()	{ return Rd6502(MC_Ix()); }
    inline BYTE MR_Iy()	{ return Rd6502(MC_Iy()); }
    
public:
	m6502cpu() : ICount(0), IRQPending(false)
	{
		// Do not reset here, as the memory
		// has probably not yet been initialized.
	}
    /** Reset6502() **********************************************/
    /** This function can be used to reset the registers before **/
    /** starting execution with Run6502(). It sets registers to **/
    /** their initial values.                                   **/
    /*************************************************************/
	void Reset()
	{
        A = X = Y = 0x00;
        P = Z_FLAG|R_FLAG;
        S = 0xFF;
        PC.B.l = Op6502(0xFFFC);
        PC.B.h = Op6502(0xFFFD);
        ICount = 0;
        IRQPending = false;
	}
	
    /** Int6502() ************************************************/
    /** This function will generate interrupt of a given type.  **/
    /** INT_NMI will cause a non-maskable interrupt. INT_IRQ    **/
    /** will cause a normal interrupt, unless I_FLAG set in R.  **/
    /*************************************************************/
    void Int(InterruptType Type);

    void Tick();
    
    inline bool ready() { if(!ICount)return true; --ICount; return false; }
};

#endif
