#ifndef BisqPPU_h
#define BisqPPU_h

#include <cstdio> /* For NULL */
using namespace std;

#include "defs.h"
#include "clock.h"

class bisqppu
{
	friend class bisqnes;
	
    BYTE vram[0x2000];
    BYTE *vpage[8];    // 1k pages (0x400)
    BYTE *vrom;
    
    typedef BYTE natab[0x400];
    natab namtab[4];
    BYTE palette[0x20];
    
    BYTE spr_ram[256];
    
    BYTE nametable(unsigned ind, unsigned ofs) const { return namtab[ind][ofs]; }
    BYTE attrtable(unsigned ind, unsigned ofs) const { return namtab[ind][ofs|0x3C0]; }
    
    void writemem(fWORD addr, fBYTE value);
    BYTE readmem(fWORD addr) const;
	
public:
	void mapvram()
	{
		for(unsigned a=0; a<8; ++a)
			vpage[a] = &vram[a*0x400];
	}
	void resizevrom(unsigned size)
	{
		if(vrom)delete[] vrom;
		vrom = new BYTE[size];
		for(unsigned a=0; a<8; ++a)
		{
			vpage[a] = &vrom[a*0x400];
			if(a*0x400 >= size)break;
		}
	}
	BYTE *vromptr() { return &vrom[0]; }
	
public:
	unsigned dmaptr;
	WORD memptr;
private:
	BYTE reg2000;
	BYTE reg2001;
	BYTE reg2003;
	bool vblank;
	unsigned tickpos, scanlid, idle;
	bool shortline;
public:
	bisqppu() : vrom(NULL)
	{
		idle = 0;
		dmaptr = 0;
		tickpos = 0;
		scanlid = 0;
		memptr = 0;
		reg2000 = 0x80;
		shortline = false;
		
		mapvram();
	}
	~bisqppu()
	{
		if(vrom)delete[] vrom;
	}

	bool write(const fWORD addr, fBYTE value);
	bool read(const fWORD addr, BYTE &value);
	
	/* We do it *exactly* like NES, no matter how slow it is */
	inline bool tick(fWORD count)
	{
		bool did_nmi = false;
    	for(; count>0; --count)
    	{
        	if(idle >= count)
        	{
        		idle -= count;
        		break;
        	}
        	// Idle left less than what we are processing
        	count -= idle;
        	idle = 0;
        	
        	if(!scanlid)
        	{
        		// No screen update
        		scanlid = 240;
        		idle = 240 * 1364;
        		continue;
        	}
        	/*
        	if(scanlid < 240)
        	{
        		if(tickpos < 1360) // 170*2
        		{
        			if(!(tickpos & 3)) // kukin nist sy nelj ticki
                	{
                		if(tickpos < 128*8)
                		{
                			// Draw background 8 pix
                		}
                		else
                		{
                			// Fetch sprite for next scanline
                		}
                	}
                	++tickpos;
                }
                else
                {
                	tickpos = 0;
                	++scanlid;
           	    	idle = 4;
           		   	// 4 cycles of idle here (HBlank).
           		   	if(scanlid==240)
           		   	{
           		   		// One idle scanline is true NES
           		   		idle += 1364;
           		   	}
            	}
            	continue;
        	}
        	*/
        	if(scanlid == 240) // One idle scanline is true NES
        	{
        		tickpos = 0;
        		scanlid = 241;
        		
        		idle = (Clock->ScanlineCount-240) * 1364;
        		if(shortline)idle -= 4;
        			
        		fprintf(stderr, "\33[0;31mVB\33[m");
        		fflush(stderr);
        		vblank = true;
        		if(reg2000 & 0x80)
        		{
        			fprintf(stderr, "\33[0;1;31mN\33[m");
        			fflush(stderr);
        			did_nmi = true;
        		}
        	}
        	else
        	{
	        	// if(scanlid == 241) - essentially true
    	    	scanlid = 0;
        		vblank = false;
        		shortline = !shortline;
        	}
        }
        return did_nmi;
	}
};

#endif
